from pywoo.utils.models import ApiObject
from pywoo.utils.parse import to_dict, ClassParser


@ClassParser(url_class="zones")
class ShippingZone(ApiObject):
    ro_attributes = {'id'}
    rw_attributes = {'name', 'order'}

    @classmethod
    def get_shipping_zone(cls, api, id=''):
        return api.get_shipping_zones(id)

    @classmethod
    def create_shipping_zone(cls, api, **kwargs):
        return api.create_shipping_zone(**kwargs)

    @classmethod
    def edit_shipping_zone(cls, api, id, **kwargs):
        return api.update_shipping_zone(id, **kwargs)

    @classmethod
    def delete_shipping_zone(cls, api, id):
        return api.delete_shipping_zone(id)

    def update(self):
        return self._api.update_shipping_zone(**to_dict(self))

    def delete(self):
        return self._api.delete_shipping_zone(self.id)

    def refresh(self):
        self.__dict__ = self._api.get_shipping_zones(id=self.id).__dict__



from pywoo.utils.models import ApiObject, ApiProperty
from pywoo.utils.parse import ClassParser

from pywoo.utils.parse import to_dict


@ClassParser(url_class="payment_gateways")
class PaymentGateway(ApiObject):
    ro_attributes = {'id', 'method_title', 'method_description'}
    rw_attributes = {'title', 'description', 'order', 'enabled', 'settings'}

    @classmethod
    def get_payment_gateways(cls, api, id='', **params):
        return api.get_payment_gateways(id, **params)

    @classmethod
    def edit_payment_gateway(cls, api, id, **kwargs):
        return api.update_payment_gateway(id, **kwargs)

    def update(self):
        return self._api.update_payment_gateway(**to_dict(self))

    def refresh(self):
        self.__dict__ = self._api.get_payment_gateways(id=self.id).__dict__


@ClassParser(url_class="payment_gateways")
class PaymentGatewaySetting(ApiProperty):
    ro_attributes = {'id', 'label', 'description', 'type', 'default', 'tip', 'placeholder'}
    rw_attributes = {'value'}



from re import search
from pywoo.utils.models import ApiObject, ApiProperty
from pywoo.utils.parse import to_dict, ClassParser


@ClassParser(url_class="refunds")
class Refund(ApiObject):
    ro_attributes = {'id', 'date_created', 'date_created_gmt'}
    wo_attributes = {'api_refund'}
    rw_attributes = {'amount', 'reason', 'refunded_by', 'meta_data', 'line_items'}

    @classmethod
    def get_refunds(cls, api, order_id, id='', **params):
        return api.get_refunds(order_id, id, **params)

    @classmethod
    def create_refund(cls, api, order_id, **kwargs):
        return api.create_refund(order_id, **kwargs)

    @classmethod
    def edit_refund(cls, api, order_id, id, **kwargs):
        return api.update_refund(order_id, id, **kwargs)

    @classmethod
    def delete_refund(cls, api, order_id, id):
        return api.delete_refund(order_id, id)

    def update(self):
        return self._api.update_refund(self.order_id, self.id, **to_dict(self))

    def delete(self):
        return self._api.delete_refund(self.order_id, self.id)

    def refresh(self):
        self.__dict__ = self._api.get_refunds(order_id=self.order_id, id=self.id).__dict__

    @property
    def order_id(self):
        return search(r"orders\/(\d+)\/.*", self._url).group(1)


@ClassParser(url_class="refunds")
class RefundLineItems(ApiProperty):
    ro_attributes = {'id', 'subtotal_tax', 'total', 'total_tax', 'taxes', 'meta_data', 'sku', 'price'}
    rw_attributes = {'name', 'product_id', 'variation_id', 'quantity', 'tax_class', 'subtotal'}


@ClassParser(url_class="refunds")
class RefundLineItemTax(ApiProperty):
    ro_attributes = {'id', 'total', 'subtotal'}



from pywoo.utils.exceptions import WriteOnlyException, ReadOnlyException


class ApiSuperClass(object):
    ro_attributes = set()
    wo_attributes = set()
    rw_attributes = set()

    def __init__(self, **kwargs):
        self.__dict__ = kwargs

    def __setattr__(self, key, value):
        if key not in self.ro_attributes:
            return super().__setattr__(key, value)
        else:
            raise ReadOnlyException(key)

    def __getattr__(self, item):
        if item not in self.wo_attributes:
            return super().__getattr__(item)
        else:
            raise WriteOnlyException(item)


class ApiObject(ApiSuperClass):
    def __init__(self, api, url, **kwargs):
        super().__init__(**kwargs)
        self._api = api
        self._url = url


class ApiActiveProperty(ApiSuperClass):
    def __init__(self, api, **kwargs):
        super().__init__(**kwargs)
        self._api = api


class ApiProperty(ApiSuperClass):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)


class MetaData(ApiProperty):
    ro_attributes = {'id'}
    rw_attributes = {'key', 'value'}



import json

from datetime import datetime
from pywoo.utils.models import ApiSuperClass, ApiObject, ApiProperty, ApiActiveProperty

urls_classes = {}


class ClassParser:
    def __init__(self, url_class):
        self.url_class = url_class

    def __call__(self, cls, *args, **kwargs):
        attrs = cls.ro_attributes.union(cls.rw_attributes)

        if self.url_class in urls_classes:
            urls_classes[self.url_class][frozenset(attrs)] = cls
        else:
            urls_classes[self.url_class] = {frozenset(attrs): cls}
        return cls


def find_mapping(data, api, url):
    if '_links' in data:
        del data['_links']
    cls = None

    key_url = url.rpartition('/')[-1]
    for attrs, class_ in urls_classes[key_url].items():
        if attrs.issubset(set(data.keys())):
            cls = class_
            break

    for key in data.keys():
        if key.startswith("date") and isinstance(data[key], str):
            data[key] = parse_date_time(data[key])

    if cls:
        if issubclass(cls, ApiObject):
            return cls(api, url, **data)
        elif issubclass(cls, ApiActiveProperty):
            return cls(api, **data)
        elif issubclass(cls, ApiProperty):
            return cls(**data)
    else:
        return data


def to_dict(data):
    if issubclass(type(data), ApiSuperClass):
        dict_data = data.__dict__
        return {key: to_dict(value) for key, value in dict_data.items() if
                not key.startswith("_") and value is not None}
    elif type(data) == datetime:
        return data.strftime('%Y-%m-%dT%H:%M:%S')
    elif type(data) == list:
        return [to_dict(d) for d in data]
    elif type(data) == dict:
        return {key: to_dict(value) for key, value in data.items()}
    return data


def parse_date_time(date_time):
    if date_time and isinstance(date_time, str):
        for pattern in ['%Y-%m-%dT%H:%M:%S', '%Y-%m-%d %H:%M:%S.%f']:
            try:
                return datetime.strptime(date_time, pattern)
            except ValueError:
                pass
    return None

def from_json(data, api, url):
    return json.loads(data, object_hook=lambda d: find_mapping(d, api, url))



from pywoo.utils.models import ApiObject
from pywoo.utils.parse import to_dict, ClassParser


@ClassParser(url_class="taxes")
class TaxRate(ApiObject):
    ro_attributes = {'id'}
    rw_attributes = {'country', 'state', 'postcode', 'city', 'rate', 'name', 'priority', 'compound', 'shipping',
                     'order', 'class'}

    @classmethod
    def get_tax_rates(cls, api, id='', **params):
        return api.get_tax_rates(id, **params)

    @classmethod
    def create_tax_rate(cls, api, **kwargs):
        return api.create_tax_rate(**kwargs)

    @classmethod
    def edit_tax_rate(cls, api, id, **kwargs):
        return api.update_tax_rate(id, **kwargs)

    @classmethod
    def delete_tax_rate(cls, api, id):
        return api.delete_tax_rate(id)

    def update(self):
        return self._api.update_tax_rate(**to_dict(self))

    def delete(self):
        return self._api.delete_tax_rate(self.id)

    def refresh(self):
        self.__dict__ = self._api.get_tax_rates(id=self.id).__dict__



from re import search

from pywoo.utils.models import ApiObject
from pywoo.utils.parse import to_dict, ClassParser


@ClassParser(url_class="terms")
class ProductAttributeTerm(ApiObject):
    ro_attributes = {'id', 'count'}
    rw_attributes = {'name', 'slug', 'description', 'menu_order'}

    @classmethod
    def get_product_attribute_terms(cls, api, product_attribute_id, id='', **params):
        return api.get_product_attribute_terms(product_attribute_id, id, **params)

    @classmethod
    def create_product_attribute_term(cls, api, product_attribute_id, **kwargs):
        return api.create_product_attribute_term(product_attribute_id, **kwargs)

    @classmethod
    def edit_product_attribute_term(cls, api, product_attribute_id, id, **kwargs):
        return api.update_product_attribute_term(product_attribute_id, id, **kwargs)

    @classmethod
    def delete_product_attribute_term(cls, api, product_attribute_id, id):
        return api.delete_product_attribute_term(product_attribute_id, id)

    def update(self):
        return self._api.update_product_attribute_term(self.product_attribute_id, **to_dict(self))

    def delete(self):
        return self._api.delete_product_attribute_term(self.product_attribute_id, self.id)

    def refresh(self):
        self.__dict__ = self._api.get_product_attribute_terms(product_attribute_id=self.product_attribute_id, id=self.id).__dict__

    @property
    def product_attribute_id(self):
        return search(r"products\/attributes\/(\d+)\/.*", self._url).group(1)



from re import search

from pywoo.utils.models import ApiObject, ApiProperty
from pywoo.utils.parse import to_dict, ClassParser


@ClassParser(url_class="variations")
class ProductVariation(ApiObject):
    ro_attributes = {'id', 'date_created', 'date_created_gmt', 'date_modified', 'date_modified_gmt', 'permalink',
                     'price', 'on_sale', 'purchasable', 'backorders_allowed', 'backordered', 'shipping_class_id'}
    rw_attributes = {'description', 'sku', 'regular_price', 'sale_price', 'date_on_sale_from',
                     'date_on_sale_from_gmt', 'date_on_sale_to', 'date_on_sale_to_gmt', 'virtual',
                     'downloadable', 'downloads', 'download_limit', 'download_expiry', 'tax_status', 'tax_class',
                     'manage_stock', 'stock_quantity', 'backorders', 'weight', 'dimensions',
                     'shipping_class', 'image', 'attributes', 'menu_order', 'meta_data'}

    @classmethod
    def get_product_variations(cls, api, product_id, id='', **params):
        return api.get_product_variations(product_id, id, **params)

    @classmethod
    def create_product_variation(cls, api, product_id, **kwargs):
        return api.create_product_variation(product_id, **kwargs)

    @classmethod
    def edit_product_variation(cls, api, product_id, id, **kwargs):
        return api.update_product_variation(product_id, id, **kwargs)

    @classmethod
    def delete_product_variation(cls, api, product_id, id):
        return api.delete_product_variation(product_id, id)

    def update(self):
        return self._api.update_product_variation(self.product_id, **to_dict(self))

    def delete(self):
        return self._api.delete_product_variation(self.product_id, self.id)

    def refresh(self):
        self.__dict__ = self._api.get_product_variations(product_id=self.product_id, id=self.id).__dict__

    @property
    def product_id(self):
        return search(r"products\/(\d+)\/.*", self._url).group(1)


@ClassParser(url_class="variations")
class ProductVariationDownload(ApiProperty):
    rw_attributes = {'id', 'name', 'file'}


@ClassParser(url_class="variations")
class ProductVariationDimension(ApiProperty):
    rw_attributes = {'lenght', 'width', 'height'}


@ClassParser(url_class="variations")
class ProductImage(ApiProperty):
    ro_attributes = {'date_created', 'date_created_gmt', 'date_modified', 'date_modified_gmt'}
    rw_attributes = {'id', 'src', 'name', 'alt'}


@ClassParser(url_class="variations")
class ProductVariationAttribute(ApiProperty):
    rw_attributes = {'id', 'name', 'option'}



from pywoo.utils.models import ApiObject
from pywoo.utils.parse import ClassParser


@ClassParser(url_class="shipping_methods")
class ShippingMethod(ApiObject):
    ro_attributes = {'id', 'title', 'description'}

    @classmethod
    def get_shipping_methods(cls, api, id='', **params):
        return api.get_shipping_methods(id, **params)

    def refresh(self):
        self.__dict__ = self._api.get_shipping_methods(id=self.id).__dict__



from pywoo.utils.models import ApiObject
from pywoo.utils.parse import ClassParser


@ClassParser(url_class="classes")
class TaxClass(ApiObject):
    ro_attributes = {'slug'}
    rw_attributes = {'name'}

    @classmethod
    def get_tax_classes(cls, api):
        return api.get_tax_classes()

    @classmethod
    def create_tax_class(cls, api, **kwargs):
        return api.create_tax_class(**kwargs)

    @classmethod
    def delete_tax_class(cls, api, slug):
        return api.delete_tax_class(slug)

    def delete(self):
        return self._api.delete_tax_class(self.slug)



from pywoo.utils.models import ApiObject, ApiProperty
from pywoo.utils.parse import to_dict, ClassParser


@ClassParser(url_class="categories")
class ProductCategory(ApiObject):
    ro_attributes = {'id', 'count'}
    rw_attributes = {'name', 'slug', 'parent', 'description', 'display', 'image', 'menu_order'}

    @classmethod
    def get_product_categories(cls, api, id='', **params):
        return api.get_product_categories(id, **params)

    @classmethod
    def create_product_category(cls, api, **kwargs):
        return api.create_product_category(**kwargs)

    @classmethod
    def edit_product_category(cls, api, id, **kwargs):
        return api.update_product_category(id, **kwargs)

    @classmethod
    def delete_product_category(cls, api, id):
        return api.delete_product_category(id)

    def update(self):
        return self._api.update_product_category(**to_dict(self))

    def delete(self):
        return self._api.delete_product_category(self.id)

    def refresh(self):
        self.__dict__ = self._api.get_product_categories(id=self.id).__dict__


@ClassParser(url_class="categories")
class ProductCategoryImage(ApiProperty):
    ro_attributes = {'date_created', 'date_created_gmt', 'date_modified', 'date_modified_gmt'}
    rw_attributes = {'id', 'src', 'name', 'alt'}



from pywoo.utils.models import ApiObject
from pywoo.utils.parse import to_dict, ClassParser


@ClassParser(url_class="reviews")
class ProductReview(ApiObject):
    ro_attributes = {'id', 'date_created', 'date_created_gmt'}
    rw_attributes = {'product_id', 'status', 'reviewer', 'reviewer_email', 'review', 'rating', 'verified'}

    @classmethod
    def get_product_reviews(cls, api, id='', **params):
        return api.get_product_reviews(id, **params)

    @classmethod
    def create_product_review(cls, api, **kwargs):
        return api.create_product_review(**kwargs)

    @classmethod
    def edit_product_review(cls, api, id, **kwargs):
        return api.update_product_review(id, **kwargs)

    @classmethod
    def delete_product_review(cls, api, id):
        return api.delete_product_review(id)

    def update(self):
        return self._api.update_product_review(**to_dict(self))

    def delete(self):
        return self._api.delete_product_review(self.id)

    def refresh(self):
        self.__dict__ = self._api.get_product_reviews(id=self.id).__dict__



from pywoo.utils.models import ApiObject, ApiProperty
from pywoo.utils.parse import to_dict, ClassParser


@ClassParser(url_class="orders")
class Order(ApiObject):
    ro_attributes = {'id', 'number', 'order_key', 'created_via', 'version', 'date_created',
                     'date_modified', 'discount_total', 'discount_tax', 'shipping_total',
                     'shipping_tax', 'cart_tax', 'total', 'total_tax', 'prices_include_tax', 'customer_ip_address',
                     'customer_user_agent', 'date_paid', 'date_completed',
                     'cart_hash', 'tax_lines', 'refunds'}
    wo_attributes = {'set_paid'}
    rw_attributes = {'parent_id', 'status', 'currency', 'customer_id', 'customer_note', 'billing', 'shipping',
                     'payment_method', 'payment_method_title', 'transaction_id', 'line_items',
                     'shipping_lines', 'fee_lines', 'coupon_lines'}

    @classmethod
    def get_orders(cls, api, id='', **params):
        return api.get_orders(id, **params)

    @classmethod
    def create_order(cls, api, **kwargs):
        return api.create_order(**kwargs)

    @classmethod
    def edit_order(cls, api, id, **kwargs):
        return api.update_order(id, **kwargs)

    @classmethod
    def delete_order(cls, api, id, **params):
        return api.delete_order(id, **params)

    def update(self):
        return self._api.update_order(**to_dict(self))

    def delete(self):
        return self._api.delete_order(self.id)

    def refresh(self):
        self.__dict__ = self._api.get_orders(id=self.id).__dict__


@ClassParser(url_class="orders")
class OrderBilling(ApiProperty):
    rw_attributes = {'first_name', 'last_name', 'company', 'address_1', 'address_2', 'city', 'state', 'postcode',
                     'country', 'email', 'phone'}


@ClassParser(url_class="orders")
class OrderShipping(ApiProperty):
    rw_attributes = {'first_name', 'last_name', 'company', 'address_1', 'address_2', 'city', 'state', 'postcode',
                     'country'}


@ClassParser(url_class="orders")
class OrderLineItems(ApiProperty):
    ro_attributes = {'id', 'subtotal_tax', 'total', 'total_tax', 'taxes', 'meta_data', 'sku', 'price'}
    rw_attributes = {'name', 'product_id', 'variation_id', 'quantity', 'tax_class', 'subtotal'}


@ClassParser(url_class="orders")
class OrderShippingLine(ApiProperty):
    ro_attributes = {'id', 'total_tax', 'taxes'}
    rw_attributes = {'method_title', 'method_id', 'total', 'meta_data'}


@ClassParser(url_class="orders")
class OrderFeeLine(ApiProperty):
    ro_attributes = {'id', 'total_tax', 'taxes'}
    rw_attributes = {'name', 'tax_class', 'tax_status', 'total', 'meta_data'}


@ClassParser(url_class="orders")
class OrderCouponLine(ApiProperty):
    ro_attributes = {'id', 'discount', 'discount_tax'}
    rw_attributes = {'code', 'meta_data'}


@ClassParser(url_class="orders")
class OrderRefund(ApiProperty):
    ro_attributes = {'id', 'reason', 'total'}


@ClassParser(url_class="orders")
class OrderTax(ApiProperty):
    ro_attributes = {'id', 'rate_code', 'rate_id', 'label', 'compound', 'tax_total', 'shipping_tax_total'}
    rw_attributes = {'meta_data'}



from pywoo.utils.models import ApiObject, ApiProperty, ApiActiveProperty
from pywoo.utils.parse import to_dict, ClassParser


class ProductCategory(ApiActiveProperty):
    rw_attributes = {'id', 'name', 'slug'}

    def get_category(self):
        self._api.get_product_categories(self.id)


class ProductTag(ApiActiveProperty):
    rw_attributes = {'id', 'name', 'slug'}

    def get_tag(self):
        self._api.get_product_tags(self.id)


@ClassParser(url_class="products")
class Product(ApiObject):
    ro_attributes = {'id', 'permalink',
                     'price', 'price_html', 'on_sale', 'purchasable', 'total_sales', 'backorders_allowed',
                     'backordered', 'shipping_required', 'shipping_taxable', 'shipping_class_id', 'average_rating',
                     'rating_count', 'related_ids', 'variations'}
    rw_attributes = {'name', 'slug', 'type', 'status', 'featured', 'catalog_visibility', 'description',
                     'short_description', 'sku', 'regular_price', 'sale_price', 'virtual', 'downloadable',
                     'downloads', 'download_limit', 'download_expiry', 'external_url', 'button_text', 'tax_status',
                     'tax_class', 'manage_stock', 'stock_quantity', 'backorders', 'sold_individually',
                     'weight', 'dimensions', 'shipping_class', 'reviews_allowed', 'upsell_ids', 'cross_sell_ids',
                     'parent_id', 'purchase_note', 'categories', 'tags', 'images', 'attributes', 'default_attributes',
                     'grouped_products', 'menu_order'}

    # noinspection PyArgumentList
    def __init__(self, api, url, **kwargs):
        super().__init__(api, url, **kwargs)
        categories = []
        tags = []
        for cat in self.categories:
            categories.append(ProductCategory(api, **cat))
        for tag in self.tags:
            tags.append(ProductTag(api, **tag))
        self.categories = categories
        self.tags = tags

    @classmethod
    def get_products(cls, api, id='', **params):
        return api.get_products(id, **params)

    @classmethod
    def create_product(cls, api, **kwargs):
        return api.create_product(**kwargs)

    @classmethod
    def edit_product(cls, api, id, **kwargs):
        return api.update_product(id, **kwargs)

    @classmethod
    def delete_product(cls, api, id, **params):
        return api.delete_product(id, **params)

    def update(self):
        return self._api.update_product(**to_dict(self))

    def delete(self):
        return self._api.delete_product(self.id)

    def refresh(self):
        self.__dict__ = self._api.get_products(id=self.id).__dict__


@ClassParser(url_class="products")
class ProductDownload(ApiProperty):
    rw_attributes = {'id', 'name', 'file'}


@ClassParser(url_class="products")
class ProductDimension(ApiProperty):
    rw_attributes = {'length', 'width', 'height'}


@ClassParser(url_class="products")
class ProductImage(ApiProperty):
    ro_attributes = {'date_created', 'date_created_gmt', 'date_modified', 'date_modified_gmt'}
    rw_attributes = {'id', 'src', 'name', 'alt'}


@ClassParser(url_class="products")
class ProductAttribute(ApiActiveProperty):
    rw_attributes = {'id', 'name', 'position', 'visible', 'variation', 'options'}

    def get_product_attribute(self):
        if self.id == 0:
            return None
        return self._api.get_product_attributes(self.id)


@ClassParser(url_class="products")
class ProductDefaultAttribute(ApiActiveProperty):
    rw_attributes = {'id', 'name', 'option'}

    def get_product_attribute(self):
        if self.id == 0:
            return None
        return self._api.get_product_attributes(self.id)



from pywoo.utils.models import ApiProperty
from pywoo.utils.parse import ClassParser


@ClassParser(url_class="locations")
class ShippingZoneLocation(ApiProperty):
    rw_attributes = {'code', 'type'}

    @classmethod
    def get_shipping_zone_locations(cls, api, shipping_zone_id):
        return api.get_shipping_zone_locations(shipping_zone_id)

    @classmethod
    def edit_shipping_zone_locations(cls, api, shipping_zone_id, shipping_locations=[]):
        return api.update_shipping_zone_locations(shipping_zone_id, shipping_locations)

    def refresh(self):
        self.__dict__ = self._api.get_shipping_zone_locations(id=self.id).__dict__



from pywoo.utils.models import ApiObject
from pywoo.utils.parse import to_dict, ClassParser


@ClassParser(url_class="coupons")
class Coupon(ApiObject):
    ro_attributes = {'id', 'date_created', 'date_modified', 'usage_count',
                     'used_by'}
    rw_attributes = {'code', 'amount', 'discount_type', 'product_ids', 'usage_limit', 'usage_limit_per_user',
                     'limit_usage_to_x_items', 'free_shipping', 'product_categories', 'excluded_product_categories',
                     'exclude_sale_items', 'minimum_amount', 'maximum_amount', 'email_restrictions'}

    @classmethod
    def get_coupons(cls, api, id='', **params):
        return api.get_coupons(id, **params)

    @classmethod
    def create_coupon(cls, api, **kwargs):
        return api.create_coupon(**kwargs)

    @classmethod
    def edit_coupon(cls, api, id, **kwargs):
        return api.update_coupon(id, **kwargs)

    @classmethod
    def delete_coupon(cls, api, id):
        return api.delete_coupon(id)

    def update(self):
        return self._api.update_coupon(**to_dict(self))

    def delete(self):
        return self._api.delete_coupon(self.id)

    def refresh(self):
        self.__dict__ = self._api.get_coupons(id=self.id).__dict__



import requests

from requests_oauthlib import OAuth1
from pywoo.models import * # NOQA
from pywoo.utils.parse import from_json
from pywoo.utils.auth import auth_params


class Api:
    def __init__(self, url, consumer_key, consumer_secret, console_logs=False):
        self.url = url
        self.consumer_key = consumer_key
        self.consumer_secret = consumer_secret
        self.console_logs = console_logs

    def _get_default_headers(self):
        headers = {}
        return headers

    def _get_auth_params(self):
        return {
            'consumer_key': self.consumer_key,
            'consumer_secret': self.consumer_secret
        } if auth_params.get(self.url.split('/')[-1], None) else {}

    def _create(self, url, data):
        resp = requests.post(
            f'{self.url}/{url}',
            json=data,
            headers=self._get_default_headers(),
            params=self._get_auth_params(),
            auth=OAuth1(self.consumer_key, self.consumer_secret, '', '')
        )
        if not resp.ok:
            raise Exception(f"\033[1;31;40mHTTP ERROR {resp.status_code} {resp.json()['message']}\033[0m")
        if self.console_logs:
            print("\033[1;32;40m[Pywoo] STATUS 200 Created successfully\033[0m")
        return from_json(resp.text, self, url)

    def _get(self, url, id='', params={}):
        resp = requests.get(
            f'{self.url}/{url}/{id}',
            params={**params, **self._get_auth_params()},
            auth=OAuth1(self.consumer_key, self.consumer_secret, '', '')
        )
        if not resp.ok:
            raise Exception(f"\033[1;31;40mHTTP ERROR {resp.status_code} {resp.json()['message']}\033[0m")
        if self.console_logs:
            print("\033[1;32;40m[Pywoo] STATUS 200 Read successfully\033[0m")
        return from_json(resp.text, self, url)

    def _put(self, url, id, data):
        resp = requests.put(
            f'{self.url}/{url}/{id}',
            json=data,
            params=self._get_auth_params(),
            auth=OAuth1(self.consumer_key, self.consumer_secret, '', '')
        )
        if not resp.ok:
            raise Exception(f"\033[1;31;40mHTTP ERROR {resp.status_code} {resp.json()['message']}\033[0m")
        if self.console_logs:
            print("\033[1;32;40m[Pywoo] STATUS 200 Updated successfully\033[0m")
        return from_json(resp.text, self, url)

    def _delete(self, url, id, params={}):
        resp = requests.delete(
            f'{self.url}/{url}/{id}',
            headers=self._get_default_headers(),
            params={**params, **self._get_auth_params()},
            auth=OAuth1(self.consumer_key, self.consumer_secret, '', '')
        )
        if not resp.ok:
            raise Exception(f"\033[1;31;40mHTTP ERROR {resp.status_code} {resp.json()['message']}\033[0m")
        if self.console_logs:
            print("\033[1;32;40m[Pywoo] STATUS 200 Deleted successfully\033[0m")
        return from_json(resp.text, self, url)

    def create_coupon(self, **data):
        return self._create('coupons', data)

    def get_coupons(self, id='', **params):
        return self._get('coupons', id, params)

    def update_coupon(self, id, **data):
        return self._put('coupons', id, data)

    def delete_coupon(self, id):
        return self._delete('coupons', id, {'force': True})

    def create_customer(self, **data):
        return self._create('customers', data)

    def get_customers(self, id='', **params):
        return self._get('customers', id, params)

    def update_customer(self, id, **data):
        return self._put('customers', id, data)

    def delete_customer(self, id, **params):
        params['force'] = True
        return self._delete('customers', id, params)

    def create_order(self, **data):
        return self._create('orders', data)

    def get_orders(self, id='', **params):
        return self._get('orders', id, params)

    def update_order(self, id, **data):
        return self._put('orders', id, data)

    def delete_order(self, id, **params):
        return self._delete('orders', id, params)

    def create_order_note(self, order_id, **data):
        return self._create(f'orders/{order_id}/notes', data)

    def get_order_notes(self, order_id, id='', **params):
        return self._get(f'orders/{order_id}/notes', id, params)

    def delete_order_note(self, order_id, id):
        return self._delete(f'orders/{order_id}/notes', id, {'force': True})

    def create_refund(self, order_id, **data):
        return self._create(f'orders/{order_id}/refunds', data)

    def get_refunds(self, order_id, id='', **params):
        return self._get(f'orders/{order_id}/refunds', id, params)

    def update_refund(self, order_id, id, **data):
        return self._put(f'orders/{order_id}/refunds', id, data)

    def delete_refund(self, order_id, id):
        return self._delete(f'orders/{order_id}/refunds', id, {'force': True})

    def create_product(self, **data):
        return self._create('products', data)

    def get_products(self, id='', **params):
        return self._get('products', id, params)

    def update_product(self, id, **data):
        return self._put('products', id, data)

    def delete_product(self, id, **params):
        return self._delete('products', id, params)

    def create_product_variation(self, product_id, **data):
        return self._create(f'products/{product_id}/variations', data)

    def get_product_variations(self, product_id, id='', **params):
        return self._get(f'products/{product_id}/variations', id, params)

    def update_product_variation(self, product_id, id, **data):
        return self._put(f'products/{product_id}/variations', id, data)

    def delete_product_variation(self, product_id, id):
        return self._delete(f'products/{product_id}/variations', id, {'force': True})

    def create_product_attribute(self, **data):
        return self._create('products/attributes', data)

    def get_product_attributes(self, id='', **params):
        return self._get('products/attributes', id, params)

    def update_product_attribute(self, id, **data):
        return self._put('products/attributes', id, data)

    def delete_product_attribute(self, id):
        return self._delete('products/attributes', id, {'force': True})

    def create_product_attribute_term(self, product_attribute_id, **data):
        return self._create(f'products/attributes/{product_attribute_id}/terms', data)

    def get_product_attribute_terms(self, product_attribute_id, id='', **params):
        return self._get(f'products/attributes/{product_attribute_id}/terms', id, params)

    def update_product_attribute_term(self, product_attribute_id, id, **data):
        return self._put(f'products/attributes/{product_attribute_id}/terms', id, data)

    def delete_product_attribute_term(self, product_attribute_id, id):
        return self._delete(f'products/attributes/{product_attribute_id}/terms', id, {'force': True})

    def create_product_category(self, **data):
        return self._create('products/categories', data)

    def get_product_categories(self, id='', **params):
        return self._get('products/categories', id, params)

    def update_product_category(self, id, **data):
        return self._put('products/categories', id, data)

    def delete_product_category(self, id):
        return self._delete('products/categories', id, {'force': True})

    def create_product_shipping_class(self, **data):
        return self._create('products/shipping_classes', data)

    def get_product_shipping_classes(self, id='', **params):
        return self._get('products/shipping_classes', id, params)

    def update_product_shipping_class(self, id, **data):
        return self._put('products/shipping_classes', id, data)

    def delete_product_shipping_class(self, id):
        return self._delete('products/shipping_classes', id, {'force': True})

    def create_product_tag(self, **data):
        return self._create('products/tags', data)

    def get_product_tags(self, id='', **params):
        return self._get('products/tags', id, params)

    def update_product_tag(self, id, **data):
        return self._put('products/tags', id, data)

    def delete_product_tag(self, id):
        return self._delete('products/tags', id, {'force': True})

    def create_product_review(self, **data):
        return self._create('products/reviews', data)

    def get_product_reviews(self, id='', **params):
        return self._get('products/reviews', id, params)

    def update_product_review(self, id, **data):
        return self._put('products/reviews', id, data)

    def delete_product_review(self, id):
        return self._delete('products/reviews', id, {'force': True})

    def create_tax_rate(self, **data):
        return self._create('taxes', data)

    def get_tax_rates(self, id='', **params):
        return self._get('taxes', id, params)

    def update_tax_rate(self, id, **data):
        return self._put('taxes', id, data)

    def delete_tax_rate(self, id):
        return self._delete('taxes', id, {'force': True})

    def create_tax_class(self, **data):
        return self._create('taxes/classes', data)

    def get_tax_classes(self):
        return self._get('taxes/classes')

    def delete_tax_class(self, id):
        return self._delete('taxes/classes', id, {'force': True})

    def get_payment_gateways(self, id=''):
        return self._get('payment_gateways', id)

    def update_payment_gateway(self, id, **data):
        return self._put('payment_gateways', id, data)

    def create_shipping_zone(self, **data):
        return self._create('shipping/zones', data)

    def get_shipping_zones(self, id=''):
        return self._get('shipping/zones', id)

    def update_shipping_zone(self, id, **data):
        return self._put('shipping/zones', id, data)

    def delete_shipping_zone(self, id):
        return self._delete('shipping/zones', id, {'force': True})

    def get_shipping_zone_locations(self, shipping_zone_id):
        return self._get(f'shipping/zones/{shipping_zone_id}/locations')

    def update_shipping_zone_locations(self, shipping_zone_id, shipping_locations=[]):
        return self._put(f'shipping/zones/{shipping_zone_id}/locations', '', shipping_locations)

    def create_shipping_zone_method(self, shipping_zone_id, **data):
        return self._create(f'shipping/zones/{shipping_zone_id}/methods', data)

    def get_shipping_zone_methods(self, shipping_zone_id, id=''):
        return self._get(f'shipping/zones/{shipping_zone_id}/methods', id)

    def update_shipping_zone_method(self, shipping_zone_id, instance_id, **data):
        return self._put(f'shipping/zones/{shipping_zone_id}/methods', instance_id, data)

    def delete_shipping_zone_method(self, shipping_zone_id, instance_id):
        return self._delete(f'shipping/zones/{shipping_zone_id}/methods', instance_id, {'force': True})

    def get_shipping_methods(self, id=''):
        return self._get('shipping_methods', id)



from re import search

from pywoo.utils.models import ApiObject, ApiProperty
from pywoo.utils.parse import to_dict, ClassParser


@ClassParser(url_class="methods")
class ShippingZoneMethod(ApiObject):
    ro_attributes = {'instance_id', 'title', 'method_id', 'method_title', 'method_description'}
    rw_attributes = {'order', 'enabled', 'settings'}

    @classmethod
    def get_shipping_zone_methods(cls, api, shipping_zone_id, id=''):
        return api.get_shipping_zone_methods(shipping_zone_id, id)

    @classmethod
    def create_shipping_zone_method(cls, api, shipping_zone_id, **kwargs):
        return api.create_shipping_zone_method(shipping_zone_id, **kwargs)

    @classmethod
    def edit_shipping_zone_method(cls, api, shipping_zone_id, id, **kwargs):
        return api.update_shipping_zone_method(shipping_zone_id, id, **kwargs)

    @classmethod
    def delete_shipping_zone_method(cls, api, shipping_zone_id, id):
        return api.delete_shipping_zone_method(shipping_zone_id, id)

    def update(self):
        return self._api.update_shipping_zone_method(self.shipping_zone_id, **to_dict(self))

    def delete(self):
        return self._api.delete_shipping_zone_method(self.shipping_zone_id, self.instance_id)

    def refresh(self):
        self.__dict__ = self._api.get_shipping_zone_methods(shipping_zone_id=self.shipping_zone_id, id=self.id).__dict__

    @property
    def shipping_zone_id(self):
        return search(r"shipping\/zones\/(\d+)\/.*", self._url).group(1)


@ClassParser(url_class="methods")
class PaymentGatewaySetting(ApiProperty):
    ro_attributes = {'id', 'label', 'description', 'type', 'default', 'tip', 'placeholder'}
    rw_attributes = {'value'}



class ReadOnlyException(Exception):
    def __init__(self, key):
        super().__init__(f'{key} attribute is read-only')


class WriteOnlyException(Exception):
    def __init__(self, key):
        super().__init__(f'{key} attribute is write-only')



from pywoo.utils.models import ApiObject, ApiProperty
from pywoo.utils.parse import to_dict, ClassParser


@ClassParser(url_class="customers")
class Customer(ApiObject):
    ro_attributes = {'id', 'date_created', 'date_modified', 'avatar_url'}
    wo_attributes = {'password'}
    rw_attributes = {'email', 'first_name', 'last_name', 'username', 'billing', 'shipping'}

    @classmethod
    def get_customers(cls, api, id='', **params):
        return api.get_customers(id, **params)

    @classmethod
    def create_customer(cls, api, **kwargs):
        return api.create_customer(**kwargs)

    @classmethod
    def edit_customer(cls, api, id, **kwargs):
        return api.update_customer(id, **kwargs)

    @classmethod
    def delete_customer(cls, api, id, **params):
        return api.delete_customer(id, **params)

    def update(self):
        return self._api.update_customer(**to_dict(self))

    def delete(self, **params):
        return self._api.delete_customer(self.id, **params)

    def refresh(self):
        self.__dict__ = self._api.get_customers(id=self.id).__dict__


@ClassParser(url_class="customers")
class CustomerBilling(ApiProperty):
    rw_attributes = {'first_name', 'last_name', 'company', 'address_1', 'address_2', 'city', 'state', 'postcode',
                     'country', 'email', 'phone'}


@ClassParser(url_class="customers")
class CustomerShipping(ApiProperty):
    rw_attributes = {'first_name', 'last_name', 'company', 'address_1', 'address_2', 'city', 'state', 'postcode',
                     'country'}



from re import search

from pywoo.utils.models import ApiObject
from pywoo.utils.parse import ClassParser


@ClassParser(url_class="notes")
class OrderNote(ApiObject):
    ro_attributes = {'id', 'date_created'}
    rw_attributes = {'note', 'customer_note'}

    @classmethod
    def get_order_notes(cls, api, order_id, id=''):
        return api.get_order_notes(order_id, id)

    @classmethod
    def create_order_note(cls, api, order_id, **kwargs):
        return api.create_order_note(order_id, **kwargs)

    @classmethod
    def delete_order_note(cls, api, order_id, id):
        return api.delete_order_note(order_id, id)

    def delete(self):
        return self._api.delete_order_note(self.order_id, self.id)

    def refresh(self):
        self.__dict__ = self._api.get_order_notes(order_id=self.order_id, id=self.id).__dict__

    @property
    def order_id(self):
        return search(r"orders\/(\d+)\/.*", self._url).group(1)



from pywoo.utils.models import ApiObject
from pywoo.utils.parse import ClassParser, to_dict


@ClassParser(url_class="tags")
class ProductTag(ApiObject):
    ro_attributes = {'id', 'count'}
    rw_attributes = {'name', 'slug', 'description'}

    @classmethod
    def get_product_tags(cls, api, id='', **params):
        return api.get_product_tags(id, **params)

    @classmethod
    def create_product_tag(cls, api, **kwargs):
        return api.create_product_tag(**kwargs)

    @classmethod
    def edit_product_tag(cls, api, id, **kwargs):
        return api.update_product_tag(id, **kwargs)

    @classmethod
    def delete_product_tag(cls, api, id):
        return api.delete_product_tag(id)

    def update(self):
        return self._api.update_product_tag(**to_dict(self))

    def delete(self):
        return self._api.delete_product_tag(self.id)

    def refresh(self):
        self.__dict__ = self._api.get_product_tags(id=self.id).__dict__
